import time
import sys
import socket
import subprocess
import traceback

DEBUG=True

# filename looks like this: /LIGHT/light/at_klosterneuburg_1_2013_0310_135047.csv

# archive looks like this: WD+SN+"_archive"+lastmontharch+".tgz"
# e.g.: /LIGHT/light/AT_LEBMIN_3_archive_2017-04.tgz


#working directory
WD="/LIGHT/light/"
logD="/LIGHT/log/"

#station's name
SN=(socket.gethostname()).strip()
SN=SN.replace("-","_")

heartfile="Heartbeat_"+SN+".txt"

gdrivecmd="/home/pi/ressources/gdrive-linux-rpi --config /home/pi/ressources/.gdrive "

#string of today's date/time
today = time.strftime("%Y-%m-%d %H:%M:%S UTC",time.gmtime(time.time()))
print("\n------ "+today)

try:
    dummy=subprocess.check_output("date > "+logD+heartfile, shell=True)
    dummy=subprocess.check_output("ls -lahrt "+WD+" | tail >> "+logD+heartfile, shell=True)
except:
    print("LightPi internal error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)


#get file list from server
try:
    gfilelist=subprocess.check_output(gdrivecmd+" list --max 500", shell=True).split("\n")
except:
    print("GDrive communications error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)


#search for heartfile. if found update, else upload.
heartid=None
for line in gfilelist:
    if heartfile in line:
        heartid=line.split(" ")[0]
try:
    if heartid==None: dummy=subprocess.check_output(gdrivecmd+" upload "+logD+heartfile, shell=True)
    else: dummy=subprocess.check_output(gdrivecmd+" update "+heartid+" "+logD+heartfile, shell=True)
except:
    print("Heartbeat upload error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)


#check free space on drive:
try:
    gabout=subprocess.check_output(gdrivecmd+" about", shell=True)
except:
    print("GDrive communications error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)

if not (  ( float(gabout.split("\n")[2].split(" ")[1]) > 1. ) and ( (gabout.split("\n")[2].split(" ")[2]).strip() == "GB" )  ):
    print("Too little drive space error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)


sys.stdout.flush()

#############################################################################################
#
#  what follows is the upload routine for the month-zip-files.
#

#do not do this near month-edges. just because.
dom = int(time.strftime("%d",time.gmtime(time.time())))
if dom in (1,2,3,29,30,31):
    sys.exit(0) #bad hair day? exit silently.


#unique identifier for last month's archive
lastmontharch = time.strftime("_%Y-%m",time.gmtime(time.time()-30*24*3600))
archfile=SN+"_archive"+lastmontharch+".tgz"


#check if archive exists:
try:
    archive=subprocess.check_output("ls "+WD+archfile, shell=True)
    if archive.strip() == "": raise IOError #no archive? go home!
except: #file-not-found raises error, which is nice
    sys.exit(0) #no archive? go home!

for line in gfilelist:
    if archfile in line:
        sys.exit(0) #file already there? nothing to do!

#ok, upload last months file.
try:
    dummy=subprocess.check_output(gdrivecmd+" upload "+WD+archfile, shell=True)
except:
    print("Archive upload error.")
    if DEBUG: 
        traceback.print_exc()
        print("\n")
    sys.exit(1)

#made it all this way down here? great!
print("Archive successfully uploaded.")

sys.exit(0)

