import serial
import time
import datetime
import subprocess
import multiprocessing

#
# interesting: if gps-time is found, the script tries to update a raspi-hw-clock
#

DEBUG=True

print("\n\n\nStarted =============================",time.ctime())


#device= "/dev/ttyUSB0"


def checkfolder(folderpath,create=True):
    import os
    """ if folderpath does not exist, folderpath will be made exist ;-)
    only works for one folder - recursive directory creation not supported 
    if create=False, only the existence of a path will be retuned
    """
    if create==False: 
        return os.path.isdir(folderpath)
    if not os.path.isdir(folderpath):
        os.mkdir(folderpath)
    return True

def timestringnow(formatstr="%Y%m%d-%H%M%S"):
    """ get time as string. new format-keyword: 'nice' """
    if formatstr=="nice": formatstr="%Y-%m-%d-%H:%M:%S"
    import datetime
    return datetime.datetime.now().strftime(formatstr)


def timestringtounix(timestr,formatstr="%Y%m%d-%H%M%S"):
    """takes time as string, returns unix time as integer
       input format is expected %Y%m%d-%H%M%S, can be specified otherwise"""
    #import time
    #return int(time.strftime("%s", time.strptime(timestr,formatstr) ))

    import datetime
    dt=datetime.datetime.strptime(timestr,formatstr)
    return (dt - datetime.datetime(1970, 1, 1, 0, 0, 0)).total_seconds() / datetime.timedelta(seconds=1).total_seconds()


def unixtotimestring(unixtime,formatstr="%Y%m%d-%H%M%S"):
    """takes unixtime as str or int, returns time string (format string optional)"""
    #import time
    ##old##return time.strftime(formatstr, time.strptime(str(unixtime),"%s") )
    #return time.strftime(formatstr, time.gmtime(float(unixtime)) )
    
    import datetime
    datetime.datetime.utcfromtimestamp(unixtime).strftime(formatstr)








def discardbuffer(seri):
    """ this reads/deletes up to 500 lines from the serial buffer.
        if there were no uptades for at least 0.05 sec, the newest
        line (first line of new data cycle / burst) is returned"""
    t=time.time()
    a=None
    for i in range(500):
        try: #this try-except catches the goniometer timeout
            a= seri.readline()
        except: pass
        if (time.time()-t)>0.05:
            return a
        t=time.time()
    return a



def controlGPSdevice(serDEVICENAME):
    import subprocess
    import serial
    import time
    try:
        device=serDEVICENAME
    except:
        device= "/dev/ttyACM0" #default device. primarily for testing

    gps=serial.Serial(device)
    gps.readline()
    time.sleep(1)
    discardbuffer(gps)

    starttime=time.time()
    while True:
        line=gps.readline()
        line=line.decode("utf-8")

        if "RMC" == line[3:6]: #can be $GPRMC, $GNRMC, $GLRMC, $GARMC
            #from ublox-nmea-manual:
            #$xxRMC,time,status,lat,NS,lon,EW,spd,cog,date,mv,mvEW,posMode,navStatus*cs<CR><LF>
            elements   =line.split(",")
            timestring =elements[1]
            automanual =elements[2]
            latstring  =elements[3]
            latchar    =elements[4]
            lonstring  =elements[5]
            lonchar    =elements[6]
            speedknots =elements[7]
            coursedir  =elements[8]
            datestring =elements[9]
            #magdeg=elements[10]
            #magEW=elements[11]

            #check whether date/time is available
            if (len(timestring)>4) and (len(datestring)>4):
                day=datestring[0:2]
                mon=datestring[2:4]
                yr= "20"+datestring[4:6]
                hh= timestring[0:2]
                mm= timestring[2:4]
                ss= timestring[4:6]
                #keep in mind: it's all strings!
                
                #full "time-date-string":
                #fulltimedatestring="20"+datestring+"_"+timestring.split(".")[0]
                fulltimedatestring=yr+mon+day+"-"+hh+mm+ss

                print(yr,mon,day," - ",hh,mm,ss)

                #compare system-time to gps-time
                timediff=abs(time.time() - timestringtounix(fulltimedatestring))

                print("------------------------------------------")
                print("sys:",timestringnow(),"   gps:",fulltimedatestring)
                print("time difference in seconds: ",timediff)
                if timediff>1.2:
                    print("sudo date -s \""+yr+"-"+mon+"-"+day+" " +hh+":"+mm+":"+ss+"\"")
                    dateanswer=subprocess.check_output("sudo date -s \""+yr+"-"+mon+"-"+day+" " +hh+":"+mm+":"+ss+"\"",shell=True)
                    print("returns: ",dateanswer)
                    try:
                        print("sudo hwclock -w")
                        dateanswer=subprocess.check_output("sudo hwclock -w",shell=True)
                        print("returns: ",dateanswer)
                    except:
                        print("hwclock error. no hardware clock?")
                else:
                    print("clock within margins")
                print("------------------------------------------")
                break
        if time.time()-starttime>180:
            print("##########################################")
            print("sys:",timestringnow())
            print("Could not check or set time \nwithin 3 minutes. no GPS?")
            print("##########################################")
            break

    return 0




def workerfunction(device):
#    logfilename=datafolder+"LOG_"+device.split("/")[-1]+"_"+timestringnow()+".txt"
    ser=serial.Serial(device, timeout=12)
    lineA=""
    while(len(lineA)<2):
        try:
            lineA=ser.readline().decode("utf-8").strip()
        except: #timeout may have occured (goniometer)
            try:
                ser.write("get:a\n")
                lineA=ser.readline().decode("utf-8").strip()
            except: #timeout again? no goniometer then...
                print(timestringnow()+"  unknown or bad device: "+device)
                lineA="xxxx"

    #now we got our test-line
    #kill first few new lines
    discardbuffer(ser)

    devicetype="UNKNOWN"

    if ("$GN" in lineA) or ("$GP" in lineA) or ("$GL" in lineA):
        print(timestringnow()+"  Found GPS on "+device)
        ser.close()     #i know this is odd. every other function gets the serial connection passed,
        time.sleep(2)   #but here the connection is terminated and only the device name gets passed.
        controlGPSdevice(device) # well, that's life....
        devicetype= "GPS"
    else:
        pass
    return devicetype


####################################################
###### this here is main
####################################################
if __name__ == '__main__':
    devlist=[x.decode("utf-8").strip() for x in subprocess.check_output("ls -1 /dev/tty*",shell=True).split() if b'ttyUSB' in x]
    devlist.extend( [x.decode("utf-8").strip() for x in subprocess.check_output("ls -1 /dev/tty*",shell=True).split() if b'ttyACM' in x] )

    if len(devlist)<1:
        raise Exception("NO ttyUSB/ttyACM-devices found! "+timestringnow())
    else:
        print(timestringnow()+"  Found the following serial devices:")
        for device in devlist:
            print(device)

    processlist=[]
    for dev in devlist:
        p=multiprocessing.Process( target=workerfunction, args=(dev,) )
        processlist.append(p)
        p.start()
        time.sleep(1.11)

    #print(len(processlist),"processes started!")

    time.sleep(1)

    for p in processlist:
        p.join()

    print(timestringnow()+"  SERIAL DEVICES: ALL PROCESSES ENDED ")
    print(timestringnow()+"  ################################### ")



